'''
    TweeLoc is created by

    Pavlos Paraskevopoulos:
        George Mason University
        LIPADE, Paris Descartes University
        pparaske@gmu.edu

    Giovanni Pellegrini:
        University of Trento
        giovanni.pellegrini@unitn.it

    Themis Palpanas:
        Paris Descartes University
        themis@mi.parisdescartes.fr

	Copyright Paraskevopoulos, Pellegrini, Palpanas.


	This software cannot be used for commercial purposes.

	You may use the software as is, only for research purposes.
	You should acknowledge the authors by referencing the following two papers (the first describes the method and algorithms; the second describes the system and visualizations):

	Pavlos Paraskevopoulos, Themis Palpanas.
	Where has this Tweet Come From? Fast and Fine-Grained Geolocalisation of Non-Geotagged Tweets. Social Network Analysis and Mining (SNAM) Journal 6(1), pp 1-16, 2016.
	http://www.mi.parisdescartes.fr/~themisp/publications/snam16.pdf

	Pavlos Paraskevopoulos, Giovanni Pellegrini, Themis Palpanas. TweeLoc: A System for Geolocalizing Tweets at Fine-Grain. International Conference on Data Mining (ICDM), New Orleans, LA, USA, November 2017.
	http://www.mi.parisdescartes.fr/~themisp/publications/icdm17-tweeloc.pdf
'''

import pyglet


class Communication:
	def __init__(self, model):
		self.model = model

	def on_key_release(self, key, modifiers):
		if modifiers == "test":
			return
		if key == pyglet.window.key.RIGHT:
			self.model.load_next_timeslot()
		elif key == pyglet.window.key.LEFT:
			self.model.load_prev_timeslot()
		elif key == pyglet.window.key.UP:
			self.model.load_next_view()
		elif key == pyglet.window.key.DOWN:
			self.model.load_prev_view()

	def on_mouse_release(self, name, id, modifiers):
		self.model.on_mouse_release(name, id, modifiers)

	def add_infobar(self, ui_manager, view):
		self.model.add_infobar(ui_manager, view)

	def add_statusbar(self, ui_manager, text):
		self.model.add_statusbar(ui_manager, text)

	def add_visualizer(self, visualizer):
		self.visualizer = visualizer

	def pass_tweet(self, tweet):
		if self.visualizer.isAlive():
			self.visualizer.add_item_to_list(tweet)

	def show_tweet(self, tweet):
		self.model.show_tweet(tweet)


