'''
    TweeLoc is created by

    Pavlos Paraskevopoulos:
        George Mason University
        LIPADE, Paris Descartes University
        pparaske@gmu.edu

    Giovanni Pellegrini:
        University of Trento
        giovanni.pellegrini@unitn.it

    Themis Palpanas:
        Paris Descartes University
        themis@mi.parisdescartes.fr

	Copyright Paraskevopoulos, Pellegrini, Palpanas.


	This software cannot be used for commercial purposes.

	You may use the software as is, only for research purposes.
	You should acknowledge the authors by referencing the following two papers (the first describes the method and algorithms; the second describes the system and visualizations):

	Pavlos Paraskevopoulos, Themis Palpanas.
	Where has this Tweet Come From? Fast and Fine-Grained Geolocalisation of Non-Geotagged Tweets. Social Network Analysis and Mining (SNAM) Journal 6(1), pp 1-16, 2016.
	http://www.mi.parisdescartes.fr/~themisp/publications/snam16.pdf

	Pavlos Paraskevopoulos, Giovanni Pellegrini, Themis Palpanas. TweeLoc: A System for Geolocalizing Tweets at Fine-Grain. International Conference on Data Mining (ICDM), New Orleans, LA, USA, November 2017.
	http://www.mi.parisdescartes.fr/~themisp/publications/icdm17-tweeloc.pdf
'''

import Queue
import time
import os
from tweepy import Stream
from tweepy import OAuthHandler
from tweepy.streaming import StreamListener
from threading import Thread


class TweetListener(StreamListener):
	def __init__(self, stop_at, save_file):
		super(TweetListener, self).__init__()
		self.stop_at = stop_at
		self.save_file = save_file

	def on_data(self, data):
		if time.time() < self.stop_at:
			self.save_file.write(data +"\n")
			return True
		else:
			return False

	def on_status(self, status):
		print status

	def on_error(self, status):
		print(status)


class Downloader(Thread):
	def __init__(self, input_folder, ts_size, bounding_box, consumer_key, consumer_secret, access_token_key, access_token_secret):

		Thread.__init__(self)
		self.input_folder = input_folder
		self.ts_duration = ts_size
		self.bounding_box = bounding_box
		self.mail = 'alive'
		self.consumer_key = consumer_key
		self.consumer_secret = consumer_secret
		self.access_token_key = access_token_key
		self.access_token_secret = access_token_secret
		self.timestamp = time.time()
		self.file_queue = Queue.Queue()

	def run(self):
		dir = os.path.dirname(__file__)
		folder_path = os.path.join(dir, self.input_folder)
		auth = OAuthHandler(self.consumer_key, self.consumer_secret)
		auth.set_access_token(self.access_token_key, self.access_token_secret)
		start_time = time.time()
		file_name = time.strftime('%d-%m-%Y %H:%M:%S', time.localtime(start_time))

		while self.mail == 'alive':
			save_file = open(folder_path + file_name + ".json", 'a')
			try:
				listener = TweetListener(start_time + 60 * self.ts_duration, save_file)
				stream = Stream(auth, listener)
				stream.filter(locations=self.bounding_box)
				while stream.running and self.mail == 'alive':
					continue
				save_file.close()
				self.file_queue.put_nowait(folder_path + file_name + ".json")
				start_time += 60 * self.ts_duration
				file_name = time.strftime('%d-%m-%Y %H:%M:%S', time.localtime(start_time))
			except:
				print "Unable to start the streaming"
				break
		print "Downloading finished"
		return
