'''
    TweeLoc is created by

    Pavlos Paraskevopoulos:
        George Mason University
        LIPADE, Paris Descartes University
        pparaske@gmu.edu

    Giovanni Pellegrini:
        University of Trento
        giovanni.pellegrini@unitn.it

    Themis Palpanas:
        Paris Descartes University
        themis@mi.parisdescartes.fr

	Copyright Paraskevopoulos, Pellegrini, Palpanas.


	This software cannot be used for commercial purposes.

	You may use the software as is, only for research purposes.
	You should acknowledge the authors by referencing the following two papers (the first describes the method and algorithms; the second describes the system and visualizations):

	Pavlos Paraskevopoulos, Themis Palpanas.
	Where has this Tweet Come From? Fast and Fine-Grained Geolocalisation of Non-Geotagged Tweets. Social Network Analysis and Mining (SNAM) Journal 6(1), pp 1-16, 2016.
	http://www.mi.parisdescartes.fr/~themisp/publications/snam16.pdf

	Pavlos Paraskevopoulos, Giovanni Pellegrini, Themis Palpanas. TweeLoc: A System for Geolocalizing Tweets at Fine-Grain. International Conference on Data Mining (ICDM), New Orleans, LA, USA, November 2017.
	http://www.mi.parisdescartes.fr/~themisp/publications/icdm17-tweeloc.pdf
'''

import matplotlib.pyplot as plt
import matplotlib.lines as mlines
import numpy as np


def create_plots_performance(results, conf):
	for k in results.keys():
		for j in results[k].keys():
			prec = np.mean(results[k][j]['prec'])
			rec = np.mean(results[k][j]['rec'])
			f1 = np.mean(results[k][j]['f1'])

			results[k][j]['prec'] = None
			results[k][j]['rec'] = None
			results[k][j]['f1'] = None

			results[k][j]['prec'] = prec
			results[k][j]['rec'] = rec
			results[k][j]['f1'] = f1

	markers = ['o', 'v', 'D', '*', 'H', 'p', 'h', 'x',
			   '.', ',', '2', 's', '^', 'd', '<', '1', '|', '_', '>', '8', '+', '4', '3']
	line_styles = ['-', '--', '-.', ':']
	tableau20 = [(31, 119, 180), (174, 199, 232), (255, 127, 14), (255, 187, 120),
				 (44, 160, 44), (152, 223, 138), (214, 39, 40), (255, 152, 150),
				 (148, 103, 189), (197, 176, 213), (140, 86, 75), (196, 156, 148),
				 (227, 119, 194), (247, 182, 210), (127, 127, 127), (199, 199, 199),
				 (188, 189, 34), (219, 219, 141), (23, 190, 207), (158, 218, 229)]
	for i in range(len(tableau20)):
		r, g, b = tableau20[i]
		tableau20[i] = (r / 255., g / 255., b / 255.)

	res_keys = results.keys()
	res_keys_2 = results[res_keys[0]].keys()
	thresh_prec = [[0 for k in res_keys] for n in res_keys_2]
	thresh_rec = [[0 for k in res_keys] for n in res_keys_2]
	thresh_f1 = [[0 for k in res_keys] for n in res_keys_2]

	list_cuts = sorted(results.keys())
	list_thresh = sorted(results[list_cuts[0]].keys())

	for k in sorted(list(results.keys())):
		for j in sorted(list(results[k].keys())):
			prec = np.mean(results[k][j]['prec'])
			rec = np.mean(results[k][j]['rec'])
			f1 = np.mean(results[k][j]['f1'])

			k_i = (k - min(list_cuts)) / 10

			for i in range(len(list_thresh)):
				if j == list_thresh[i]:
					j_i = i
					break

			thresh_prec[j_i][k_i] = prec
			thresh_rec[j_i][k_i] = rec
			thresh_f1[j_i][k_i] = f1

	for m in range(3):
		x = np.array([x for x in range(min(list_cuts), max(list_cuts) + 1, 10)])
		plt.figure(figsize=(8, 8))
		ax = plt.subplot(111)
		ax.spines["top"].set_visible(False)
		# ax.spines["bottom"].set_visible(False)
		ax.spines["right"].set_visible(False)
		# ax.spines["left"].set_visible(False)
		ax.get_xaxis().tick_bottom()
		ax.get_yaxis().tick_left()

		plt.ylim(0, 100)
		plt.xlim(min(list_cuts), max(list_cuts) + 5)
		plt.yticks(fontsize=15)
		plt.xticks(x, [str(h) for h in range(min(list_cuts), max(list_cuts)+1, 10)], fontsize=15)
		plt.xlabel("% of keywords used", size=18)

		legend = []
		j = 0
		list_thresh_str = []
		for t in list_thresh:
			if t == -110:
				q = "No"
			elif t == -120:
				q = "QL"
			elif t == -130:
				q = "KL"
			else:
				q = str(t)
			list_thresh_str.append(q)
		if m == 0:
			plt.ylabel("% Precision", size=18)
			for i in range(len(thresh_prec)):
				plt.plot(x, np.array(thresh_prec[i]), lw=2, color=tableau20[i], linestyle=line_styles[i % 4],
						 marker=markers[i], markersize=10)
				legend.append(mlines.Line2D([], [], color=tableau20[i], marker=markers[i], markersize=14,
											label=list_thresh_str[j], linestyle=line_styles[i % 4]))
				plt.legend(handles=legend, prop={'size': 15}, loc=0)
				plt.savefig(conf.dir_path + "/Plots/precision.png")
				j += 1
		elif m == 1:
			plt.ylabel("% Recall", size=18)
			for i in range(len(thresh_rec)):
				plt.plot(x, np.array(thresh_rec[i]), lw=2, color=tableau20[i], linestyle=line_styles[i % 4],
						 marker=markers[i], markersize=10)
				legend.append(mlines.Line2D([], [], color=tableau20[i], marker=markers[i], markersize=14,
											label=list_thresh_str[j], linestyle=line_styles[i % 4]))
				plt.legend(handles=legend, prop={'size': 15}, loc=0)
				plt.savefig(conf.dir_path + "/Plots/recall.png")
				j += 1
		elif m == 2:
			plt.ylabel("% F1", size=18)
			for i in range(len(thresh_f1)):
				plt.plot(x, np.array(thresh_f1[i]), lw=2, color=tableau20[i], linestyle=line_styles[i % 4],
						 marker=markers[i], markersize=10)
				legend.append(mlines.Line2D([], [], color=tableau20[i], marker=markers[i], markersize=14,
											label=list_thresh_str[j], linestyle=line_styles[i % 4]))
				plt.legend(handles=legend, prop={'size': 15}, loc=0)
				plt.savefig(conf.dir_path + "/Plots/f1.png")
				j += 1
		plt.close()


