#!/usr/bin/python

'''
    TweeLoc is created by

    Pavlos Paraskevopoulos:
        George Mason University
        LIPADE, Paris Descartes University
        pparaske@gmu.edu

    Giovanni Pellegrini:
        University of Trento
        giovanni.pellegrini@unitn.it

    Themis Palpanas:
        Paris Descartes University
        themis@mi.parisdescartes.fr

    Copyright Paraskevopoulos, Pellegrini, Palpanas.


    This software cannot be used for commercial purposes.

    You may use the software as is, only for research purposes.
    You should acknowledge the authors by referencing the following two papers (the first describes the method and algorithms; the second describes the system and visualizations):

    Pavlos Paraskevopoulos, Themis Palpanas.
    Where has this Tweet Come From? Fast and Fine-Grained Geolocalisation of Non-Geotagged Tweets. Social Network Analysis and Mining (SNAM) Journal 6(1), pp 1-16, 2016.
    http://www.mi.parisdescartes.fr/~themisp/publications/snam16.pdf

    Pavlos Paraskevopoulos, Giovanni Pellegrini, Themis Palpanas. TweeLoc: A System for Geolocalizing Tweets at Fine-Grain. International Conference on Data Mining (ICDM), New Orleans, LA, USA, November 2017.
    http://www.mi.parisdescartes.fr/~themisp/publications/icdm17-tweeloc.pdf
'''

import sys
import preprocessing
import downloader
import time
import plots
from algorithm import TweeLocAlgorithmOffline, TweeLocAlgorithmOnline, TweeLocAlgorithmThread

if __name__ == '__main__':
    if len(sys.argv) < 3:
        print "Usage: -t online/offline."
        sys.exit(1)
    elif sys.argv[1] == '-t':
        exp_type = sys.argv[2]
    else:
        print "Usage: -t online/offline."
        sys.exit(1)
    # exp_type = 'offline'

    try:
        conf = preprocessing.check_parameters_value()
        conf.print_Conf()
    except Exception as e:
        print "Error in the settings file: {0}. Program will be closed.".format(e.__str__())
        sys.exit(1)
    preprocessing.create_folder_structure(conf)
    # print "\nTo stop the program at any moment, digit 'stop' + ENTER\n\n"

    if exp_type == 'offline':
        algorithm = TweeLocAlgorithmOffline(conf)
        algorithm_thread = TweeLocAlgorithmThread(algorithm=algorithm, configuration=conf)
        try:
            algorithm_thread.start()
        except:
            algorithm_thread.stop = True
            algorithm_thread.join()
            print "Stopping algorithm thread..."
            sys.exit(1)
        while not algorithm.work_done:
            time.sleep(1.0)
            continue
        res = algorithm_thread.get_results()
        plots.create_plots_performance(res, conf)
        algorithm.stop = True
        algorithm_thread.join()
        print "Stopping algorithm thread..."

    elif exp_type == 'online':
        downloader = downloader.Downloader(conf.folder, conf.timeslot_size, conf.bounding_box, conf.consumer_key, conf.consumer_secret, conf.access_token_key, conf.access_token_secret)
        downloader.start()

        print "\nTo stop the program at any moment, digit 'stop' + ENTER\n\n"

        algorithm = TweeLocAlgorithmOnline(conf, downloader)
        print "Download thread started. First analysis result will come after {0} minutes.\n".format(
            conf.timeslot_size * conf.window)
        algorithm_thread = TweeLocAlgorithmThread(algorithm=algorithm, configuration=conf)
        try:
            algorithm_thread.start()
        except:
            algorithm.stop = True
            downloader.mail = 'stop'
            algorithm_thread.join()
            downloader.join()
            print "Stopping algorithm thread..."
            sys.exit(1)
        while not raw_input() == 'stop':
            if not downloader.isAlive():
                downloader.join(2)
                time.sleep(1.0)
                downloader.start()
            time.sleep(0.5)
            continue
        algorithm.stop = True
        downloader.mail = 'stop'
        algorithm_thread.join()
        downloader.join()
        print "Stopping algorithm thread..."
        print "Stopping downloader thread..."
